@@ -2,11 +2,13 @@ import os
 
 import sensenet.importers
 tf = sensenet.importers.import_tensorflow()
+kl = sensenet.importers.import_keras_layers()
 
-from sensenet.constants import IMAGE_STANDARDIZERS, WARP, PAD, CROP
+from sensenet.constants import IMAGE_STANDARDIZERS, WARP, PAD, CROP, DCT
 from sensenet.accessors import get_image_shape
-from sensenet.layers.utils import constant, propagate
-from sensenet.layers.construct import layer_sequence
+from sensenet.layers.utils import constant, build_graph
+from sensenet.layers.construct import LAYER_FUNCTIONS
+from sensenet.models.settings import ensure_settings
 
 CONTRAST_LIMIT = 0.25
 EXPECTED_LOW = 255 * CONTRAST_LIMIT
@@ -33,11 +35,11 @@ def resize_with_crop_or_pad(settings, target_dims, image):
     out_dims = tf.cast(target_dims, tf.float32)
 
     box_scale = scale_for_box(in_dims, out_dims, pad_only)
-    scaled_dims = tf.cast(tf.math.round(in_dims * box_scale), tf.int32)
-    scaled = tf.image.resize(image, scaled_dims, method='nearest')
+    scaled_dims = in_dims * box_scale
+    int_dims = tf.cast(tf.math.round(scaled_dims), tf.int32)
+    scaled = tf.image.resize(image, int_dims, method='nearest')
 
     if pad_only:
-        scaled_dims = tf.cast(scaled_dims, tf.float32)
         pad_h = [0, out_dims[0] - scaled_dims[0]]
         pad_w = [0, out_dims[1] - scaled_dims[1]]
 
@@ -48,8 +50,20 @@ def resize_with_crop_or_pad(settings, target_dims, image):
 
         return tf.pad(scaled, pad, constant_values=tf.reduce_mean(scaled))
     else:
-        height, width = target_dims[0], target_dims[1]
-        return tf.image.resize_with_crop_or_pad(image, height, width)
+        scaled_height, scaled_width = scaled_dims[0], scaled_dims[1]
+        height, width = out_dims[0], out_dims[1]
+
+        hbeg = tf.cast(tf.math.round((scaled_height - height) / 2), tf.int32)
+        wbeg = tf.cast(tf.math.round((scaled_width - width) / 2), tf.int32)
+        hend = hbeg + tf.cast(height, tf.int32)
+        wend = wbeg + tf.cast(width, tf.int32)
+
+        if len(image.shape) == 3:
+            return scaled[hbeg:hend,wbeg:wend,:]
+        elif len(image.shape) == 4:
+            return scaled[:,hbeg:hend,wbeg:wend,:]
+        else:
+            raise ValueError('Image tensor is rank %d' % len(image.shape))
 
 # Unused right now, but I'll leave it here just in case
 def adjust_contrast(image):
@@ -84,95 +98,171 @@ def rescale(settings, target_shape, image):
         else:
             raise ValueError('Image tensor is rank %d' % len(image.shape))
     elif image.shape[-1] != 3:
-        raise ValueError('Number of color channels is %d' % image.shape[-1])
+        raise ValueError('Number of color channels is %d' % new_image.shape[-1])
+
+    if len(image.shape) == 4:
+        new_image.set_shape([None, None, None, 3])
+    elif len(image.shape) == 3:
+        new_image.set_shape([None, None, 3])
+    else:
+        raise ValueError('Image tensor is rank %d' % len(image.shape))
 
     if settings.color_space and settings.color_space.lower().startswith('bgr'):
         return tf.reverse(new_image, axis=[-1])
     else:
         return new_image
 
-def make_image_reader(settings, target_shape, return_original_dimensions):
-    nchannels = target_shape[-1]
+def make_image_reader(settings, target_shape):
+    n_chan = target_shape[-1]
     input_format = settings.input_image_format or 'file'
-    prefix = settings.image_path_prefix
+
+    if settings.image_path_prefix:
+        prefix = settings.image_path_prefix + os.sep
+    else:
+        prefix = ''
 
     def read_image(path_or_bytes):
         if input_format == 'pixel_values':
             raw = path_or_bytes
         else:
-            if input_format == 'file':
-                path = path_or_bytes
-
-                if prefix:
-                    path = tf.strings.join([prefix + os.sep, path])
-
-                img_bytes = tf.io.read_file(path)
-            else:
+            if input_format == 'image_bytes':
                 img_bytes = path_or_bytes
+            else:
+                path = tf.strings.join([prefix, path_or_bytes])
+                img_bytes = tf.io.read_file(path)
 
-            # Note that, spectacularly weirdly, this method will also
-            # work for pngs and gifs.  Even wierder, We can't use
-            # `decode_image` here because the tensor that comes out
-            # doesn't have a shape!
-            raw = tf.io.decode_jpeg(img_bytes,
-                                    dct_method='INTEGER_ACCURATE',
-                                    channels=nchannels)
+            raw = tf.io.decode_jpeg(img_bytes, dct_method=DCT, channels=n_chan)
 
-        if return_original_dimensions:
-            return rescale(settings, target_shape, raw), tf.shape(raw)
-        else:
-            return rescale(settings, target_shape, raw)
+        return rescale(settings, target_shape, raw)
 
     return read_image
 
-class ImageReader(tf.keras.layers.Layer):
-    def __init__(self, network, settings):
-        super(ImageReader, self).__init__()
+class ImageReaderLayer(tf.keras.layers.Layer):
+    def __init__(self, **kwargs):
+        layer_args = dict(kwargs)
+        layer_args.pop('settings')
+        layer_args.pop('input_shape')
 
-        self._input_shape = get_image_shape(network)
-        self._settings = settings
+        super().__init__(**layer_args)
 
-    def build(self, input_shape):
-        self._read = make_image_reader(self._settings, self._input_shape, False)
+        self._settings = ensure_settings(kwargs['settings'])
+        self._input_shape = kwargs['input_shape']
+        self._nchannels = self._input_shape[-1]
+        self._prefix = self._settings.image_path_prefix or '.'
 
     def call(self, inputs):
+        return tf.map_fn(self.read, inputs, fn_output_signature=tf.uint8)
+
+    def get_config(self):
+        config = super().get_config()
+
+        config.update({
+            'settings': dict(vars(self._settings)),
+            'input_shape': list(self._input_shape)
+        })
+
+        return config
+
+class ImageFileReaderLayer(ImageReaderLayer):
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+
+    def read(self, path):
+        path = tf.strings.join([self._prefix + os.sep, path])
+        img = tf.io.read_file(path)
+
+        # Note that, spectacularly weirdly, this method will also
+        # work for pngs and gifs.  Even wierder, We can't use
+        # `decode_image` here because the tensor that comes out
+        # doesn't have a shape!
+        raw = tf.io.decode_jpeg(img, dct_method=DCT, channels=self._nchannels)
+
+        return rescale(self._settings, self._input_shape, raw)
+
+class ImageBytesReaderLayer(ImageReaderLayer):
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+
+    def read(self, img):
+        raw = tf.io.decode_jpeg(img, dct_method=DCT, channels=self._nchannels)
+        return rescale(self._settings, self._input_shape, raw)
+
+class ImageShapeReaderLayer(ImageReaderLayer):
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+
+    def read(self, path):
+        path = tf.strings.join([self._prefix + os.sep, path])
+        img = tf.io.read_file(path)
+        raw = tf.io.decode_jpeg(img, dct_method=DCT, channels=self._nchannels)
+
+        return rescale(self._settings, self._input_shape, raw), tf.shape(raw)
+
+    def call(self, inputs):
+        return self.read(inputs)
+
+class ImageReader():
+    def __init__(self, network, settings):
+        self._settings = settings
+        self._input_shape = get_image_shape(network)
+
+        if self._settings.rescale_type is None:
+            net_meta = network['metadata']
+            self._settings.rescale_type = net_meta.get('rescale_type', WARP)
+
+    def reader(self, get_shape=False):
+        config = {
+            'settings': dict(vars(self._settings)),
+            'input_shape': list(self._input_shape)
+        }
+
+        if self._settings.input_image_format == 'image_bytes':
+            return ImageBytesReaderLayer(**config)
+        elif get_shape:
+            return ImageShapeReaderLayer(**config)
+        else:
+            return ImageFileReaderLayer(**config)
+
+    def __call__(self, inputs):
         if self._settings.input_image_format == 'pixel_values':
             dims = tf.constant(self._input_shape[1:3], tf.int32)
             images = rescale(self._settings, self._input_shape, inputs)
         else:
-            images = tf.map_fn(self._read, inputs, fn_output_signature=tf.uint8)
+            read_layer = self.reader()
+            images = read_layer(inputs)
 
         return tf.cast(images, tf.float32)
 
 class BoundingBoxImageReader(ImageReader):
     def __init__(self, network, settings):
         super(BoundingBoxImageReader, self).__init__(network, settings)
+        self._settings.rescale_type = PAD
 
-    def build(self, input_shape):
-        self._read = make_image_reader(self._settings, self._input_shape, True)
-
-    def call(self, inputs):
+    def __call__(self, inputs):
         if self._settings.input_image_format == 'pixel_values':
             dims = tf.constant(self._input_shape[1:3], tf.int32)
             image = rescale(self._settings, self._input_shape, inputs)
             original_dims = tf.expand_dims(tf.shape(inputs)[1:], axis=0)
         else:
+            read = self.reader(True)
+
             # We're explicitly saying here we only take one filename at
             # a time; if multiple files are passed in, all but the first
             # one are ignored.
-            image, dims = self._read(inputs[0, 0])
+            image, original_dims = read(inputs[0, 0])
+            image = tf.reshape(image, self._input_shape[1:])
+
             image = tf.expand_dims(image, axis=0)
-            original_dims = tf.expand_dims(dims, axis=0)
+            original_dims = tf.expand_dims(original_dims, axis=0)
 
         return tf.cast(image, tf.float32), original_dims
 
-class ImageLoader(tf.keras.layers.Layer):
+class ImageLoader():
     def __init__(self, network):
-        super(ImageLoader, self).__init__()
-
         metadata = network['metadata']
         method = metadata['loading_method']
-        mimg = metadata['mean_image']
+        mimg = metadata.get('mean_image', None)
+
         mean, std = IMAGE_STANDARDIZERS[method]
 
         self._reverse = method == 'channelwise_centering'
@@ -180,7 +270,7 @@ class ImageLoader(tf.keras.layers.Layer):
         self._stdev = constant(std) if std != 1 else None
         self._mean_image = constant(mimg) if mimg is not None else None
 
-    def call(self, inputs):
+    def __call__(self, inputs):
         images = inputs
 
         if self._reverse:
@@ -197,16 +287,15 @@ class ImageLoader(tf.keras.layers.Layer):
 
         return images
 
-class ImagePreprocessor(tf.keras.layers.Layer):
+class ImagePreprocessor():
     def __init__(self, image_network, settings):
-        super(ImagePreprocessor, self).__init__()
-
         self._reader = ImageReader(image_network, settings)
         self._loader = ImageLoader(image_network)
-        self._image_layers = layer_sequence(image_network)
+        self._image_layers = image_network['layers']
 
-    def call(self, inputs):
+    def __call__(self, inputs):
         raw_images = self._reader(inputs)
         images = self._loader(raw_images)
+        outputs = build_graph(self._image_layers, LAYER_FUNCTIONS, images)
 
-        return propagate(self._image_layers, images)
+        return outputs[-1].output
