@@ -1,11 +1,12 @@
+import sensenet.importers
+tf = sensenet.importers.import_tensorflow()
+
 import os
 import gzip
 import json
 
-import sensenet.importers
-tf = sensenet.importers.import_tensorflow()
-
-from sensenet.layers.utils import propagate
+from sensenet.constants import DCT
+from sensenet.preprocess.image import rescale
 
 TEST_DATA_DIR = 'tests/data/'
 TEST_IMAGE_DATA = os.path.join(TEST_DATA_DIR, 'images/')
@@ -16,8 +17,3 @@ def read_regression(path, root=TEST_DATA_DIR):
 def read_zipped_json(path):
     with gzip.open(path, "rb") as fin:
         return json.loads(fin.read().decode('utf-8'))
-
-def make_model(lseq, input_shape):
-    inputs = tf.keras.Input(input_shape, dtype=tf.float32)
-    outputs = propagate(lseq, inputs)
-    return tf.keras.Model(inputs=inputs, outputs=outputs)
